/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package cz.cvut.ds.dsjdbc;

import cz.cvut.ds.dsjdbc.entities.Department;
import cz.cvut.ds.dsjdbc.entities.Employee;
import cz.cvut.ds.dsjdbc.entities.Salary;
import java.sql.*;
import java.util.*;

/**
 *
 * @author marek
 */
public class Storage {

    private static Storage storage = null;
    private Connection con;

    protected Storage() throws ClassNotFoundException, SQLException {
        DatabaseConnection dbconn = new DatabaseConnection();
        con = dbconn.getConnection();
    }

    public static Storage getInstance() throws ClassNotFoundException, SQLException {
        if (storage == null) {
            storage = new Storage();
        }
        return storage;
    }


    public List<Department> getAllDepartments() throws SQLException {
        List<Department> list = new ArrayList<Department>();
        Statement st = con.createStatement();
        ResultSet rs = st.executeQuery("SELECT * FROM department");
        while(rs.next()) {
            list.add(loadDepartment(rs));
        }
        return list;
    }

    protected Department loadDepartment(final ResultSet rs) throws SQLException {
        Department d = new Department();
        d.setDepartmentId(rs.getInt("department_id"));
        d.setDepartmentDescription(rs.getString("department_description"));
        return d;
    }

    public List<Employee> getAllEmployees() throws SQLException {
        List<Employee> list = new ArrayList<Employee>();
        Statement st = con.createStatement();
        ResultSet rs = st.executeQuery("SELECT * FROM employee");
        while(rs.next()) {
            list.add(loadEmployee(rs));
        }
        return list;
    }

    public List<Employee> getEmployeesByDepartment(Department dep) throws SQLException {
        List<Employee> list = new ArrayList<Employee>();
        PreparedStatement ps = con.prepareStatement("SELECT * FROM employee WHERE department_id=?");
        ps.setInt(1, dep.getDepartmentId());
        ResultSet rs = ps.executeQuery();
        while(rs.next()) {
            list.add(loadEmployee(rs));
        }
        return list;
    }

    protected Employee loadEmployee(final ResultSet rs) throws SQLException {
        Employee e = new Employee();
        e.setEmployeeId(rs.getInt("employee_id"));
        e.setBirthDate(rs.getDate("birth_date"));
        e.setFullName(rs.getString("full_name"));
        e.setDepartmentId(rs.getInt("department_id"));
        return e;
    }

    public void insertEmployee(Employee emp) throws SQLException {
        PreparedStatement ps = con.prepareStatement("INSERT INTO employee (employee_id,birth_date,full_name,department_id) VALUES (?,?,?,?)");
        ps.setInt(1, emp.getEmployeeId());
        ps.setDate(2, emp.getBirthDate());
        ps.setString(3, emp.getFullName());
        ps.setInt(4, emp.getDepartmentId());
        ps.executeUpdate();
    }

    public List<Salary> getAllSalaries() throws SQLException {
        List<Salary> list = new ArrayList<Salary>();
        Statement st = con.createStatement();
        ResultSet rs = st.executeQuery("SELECT * FROM salary");
        while(rs.next()) {
            list.add(loadSalary(rs));
        }
        return list;
    }

    public List<Salary> getSalariesByEmployee(Employee emp) throws SQLException {
        List<Salary> list = new ArrayList<Salary>();
        PreparedStatement ps = con.prepareStatement("SELECT * FROM salary WHERE employee_id=?");
        ps.setInt(1, emp.getEmployeeId());
        ResultSet rs = ps.executeQuery();
        while(rs.next()) {
            list.add(loadSalary(rs));
        }
        return list;
    }

    protected Salary loadSalary(final ResultSet rs) throws SQLException {
        Salary s = new Salary();
        s.setEmployeeId(rs.getInt("employee_id"));
        s.setPayDate(rs.getDate("pay_date"));
        s.setSalaryPaid(rs.getDouble("salary_paid"));
        return s;
    }

}
